AJS.SharePoint = AJS.SharePoint || {};

(function($) {

    /**
     * Provides a convenient way to display an HTML <select> element that is automatically populated with the set of
     * SharePoint sites configured in Confluence. The contents of the field are asynchronously populated when the set of
     * sites retrieved from Confluence is returned.
     *
     * If a SharePoint site is configured with a Confluence Access URL (aka. 'Alternative Access URL'), then the
     * 'user-facing' URL is the one that will be displayed to the user.
     *
     *
     * @param   dropDown                (required) A jQuery selector object wrapping the <select> element to bind. The
     *                                  length of the selector should be 1 (ie. haven't tested binding multiple fields
     *                                  at once).
     * @param   onChange                (optional) A function callback that is invoked whenever the selected value in
     *                                  the drop-down is changed. The default callback will, when the value is changed,
     *                                  issue a GET request to Confluence that primes the CXF web service proxies in
     *                                  Confluence for the selected SharePoint site. This ensures that the proxy cache
     *                                  is 'warm', in anticipation of the user most likely being about to issue a 'real'
     *                                  query against that web service.
     * @param   initialValueSelector    (optional) A function callback that is invoked in order to select the initial
     *                                  value of the drop-down after the set of sites has been retrieved from Confluence.
     *                                  The default behaviour is to restore the original value at the time this binder
     *                                  function was invoked. If there is no original value, the site that is configured
     *                                  as the "default" will be selected.
     */
    AJS.SharePoint.bindSitesDropDown = function(dropDown, onChange, initialValueSelector) {
        // Disable the drop-down until its content has been loaded. Display a spinner to show progress.
        var spinner = showSpinner(dropDown);

        // Substitute the callback functions for their defaults, if not specified.
        onChange = onChange ? onChange : defaultOnChangeHandler;
        initialValueSelector = initialValueSelector ? initialValueSelector : defaultInitialValueSelector;

        // Retrieve sites from Confluence and update drop-down when successfully retrieved.
        AJS.SharePoint.getSitesAsync(function (sites) {
            // Handle case where nothing was returned.
            if (!sites.length) {
                AJS.log("Configured SharePoint sites result was not in the expected format.");
                spinner.remove();
                return;
            }

            // Store the drop-down's currently selected value.
            var currentValue = dropDown.val();

            // Clear out all child elements in the drop-down and replace with the SharePoint site information.
            dropDown.empty();
            dropDown.append(AJS.$("<option/>").attr("value", ""));

            var defaultSite = null;
            $.each(sites, function(index, site) {
                if (site.defaultSharepointSite)
                    defaultSite = site;

                var option = $("<option/>");
                option.attr("value", site.alias);
                option.attr("title", site.serverUrl);
                option.text(site.alias + " (" + site.serverUrl + ")");
                dropDown.append(option);
            });

            // restore the currently selected value.
            var defaultValue = initialValueSelector(currentValue, sites, defaultSite);
            if (!defaultValue)
                dropDown.val("");
            else if (!defaultValue.alias)
                dropDown.val(defaultValue);
            else
                dropDown.val(defaultValue.alias);

            // AJAX complete; ensure that the spinner is hidden and the drop-down is enabled.
            spinner.remove();
            dropDown.change(onChange);

        }, function(jqXhr, textStatus, errorThrown) {
            AJS.log(textStatus + " - " + errorThrown);
            spinner.remove();
        });
    };

    /**
     * Shows a spinning animated gif immediately adjacent to the specified element.
     *
     * @param   dropDown    a jQuery selector object wrapping the DOM element to display
     *
     * @returns a jQuery selector wrapping the inserted spinner animation (so that calling code to hide the spinner when
     *          desired.
     */
    function showSpinner(dropDown) {
        var spinnerURL = AJS.params.contextPath + "/download/resources/com.atlassian.confluence.extra.sharepoint/ajax-loader.gif";

        dropDown.addClass("disabled", "true");
        // TODO: Use a Soy template for this?
        dropDown.after("<img class=\"sharepoint-loader\" src=\"" + spinnerURL + "\">");

        return dropDown.next();
    }

    /**
     * Default onChange handler for {@link AJS.SharePoint#bindSitesDropDown} if no handler is specified. When invoked,
     * primes the Confluence web service cache for the selected SharePoint site.
     */
    function defaultOnChangeHandler() {
        var dropDown = $(this);

        // When a value is selected in the sites drop down, tell Confluence to prime the web service client cache for
        // that site, as it is now quite likely that the user is about to make use of the site. If we don't prime the
        // cache, the first attempt to do any kind of web service call against the specified site will probably probably
        // time-out, as constructing the web service proxy objects for the first time can be slow.
        var requestSettings = {
            async: true,
            url: AJS.params.contextPath + "/rest/sharepoint/1.0/search/primer?siteAlias=" + dropDown.children("option:selected").attr("value"),
            timeout: 30000, // 30 seconds.
            success: function() {
                AJS.log("SOAP Cache priming successful.");
            },
            error: function() {
                AJS.log("SOAP Cache priming failed.");
            }
        };
        AJS.$.ajax(requestSettings);
    }

    /**
     * Default initialValueSelector for {@link AJS.SharePoint#bindSitesDropDown} if no callback is specified. When
     * invoked selects the original value in the drop-down, if one was present. Otherwise, the default site is selected.
     *
     * @param   originalValue   The original value that was stored in the drop-down before the binder was called. If
     *                          the drop-down had no elements in it when the binder was called, this value will be null.
     * @param   sites           The set of configured SharePoint sites returned from Confluence. A JSON object returned
     *                          from the com.atlassian.confluence.extra.sharepoint.actions.ConfiguredSitesAction XWork
     *                          action.
     * @param   defaultSite     The site in the {@see sites} parameter that is the configured default site (ie. has
     *                          defaultSite == true).
     */
    function defaultInitialValueSelector(originalValue, sites, defaultSite) {
        if (!originalValue || originalValue == "") {
            return defaultSite;
        }
        return originalValue;
    }
})(AJS.$);